searchData={"items":[{"type":"module","title":"Oxide.Result","doc":"Helpers for working with result tuples, `{:ok, value}` and `{:error, reason}`.\n\nUnless otherwise stated, functions raise `FunctionClauseError` when given an unexpected\nnon-result.","ref":"Oxide.Result.html"},{"type":"macro","title":"Oxide.Result.&&&/2","doc":"Result pipe operator.\n\nThe result pipe operator `&&&/2` is a result-aware analogue to the pipe operator `|>`.\nIt allows chaining functions that return results, piping the inner value of `:ok` results\nand short-circuiting the pipeline if any of the functions return an error. For example\n\n    with {:ok, x1}   f1() &&& f2() &&& f3()\n\nMore examples:\n\n    iex> {:ok, :foo} &&& Atom.to_string()\n    \"foo\"\n    iex> {:ok, 3} &&& then(fn x -> {:ok, x + 1} end) &&& List.wrap()\n    [4]\n    iex> {:ok, :foo} &&& Atom.to_string() |> String.capitalize()\n    \"Foo\"\n    iex> {:error, :oops} &&& Atom.to_string() |> String.capitalize()\n    {:error, :oops}","ref":"Oxide.Result.html#&&&/2"},{"type":"function","title":"Oxide.Result.and_then/2","doc":"","ref":"Oxide.Result.html#and_then/2"},{"type":"function","title":"Oxide.Result.collect/1","doc":"Collects a list of results into a single result.\n\nIf any of the results is an error, the first error is returned. Otherwise, a single\nok result is returned with a list of the result values.\n\n    iex> [{:ok, 1}, {:ok, 2}, {:ok, 3}] |> Result.collect()\n    {:ok, [1, 2, 3]}\n    iex> [{:ok, 1}, {:error, 2}, {:ok, 3}, {:error, 4}] |> Result.collect()\n    {:error, 2}","ref":"Oxide.Result.html#collect/1"},{"type":"function","title":"Oxide.Result.err_if_nil/2","doc":"Convert a maybe-nil value to a result.\n\nMaps `nil` to `{:error, reason}` and any non-`nil` value to `{:ok, value}`.\n\n    iex> %{\"key\" => \"value\"} |> Map.get(\"key\") |> Result.err_if_nil(:notfound)\n    {:ok, \"value\"}\n    iex> %{\"key\" => \"value\"} |> Map.get(\"missing\") |> Result.err_if_nil(:notfound)\n    {:error, :notfound}","ref":"Oxide.Result.html#err_if_nil/2"},{"type":"function","title":"Oxide.Result.error/1","doc":"Wrap a value in an error result.\n\n    iex> :some_error_reason |> Result.error()\n    {:error, :some_error_reason}","ref":"Oxide.Result.html#error/1"},{"type":"function","title":"Oxide.Result.is_error?/1","doc":"Return whether a result is an error.\n\n    iex> Result.is_error?({:ok, 3})\n    false\n    iex> Result.is_error?({:error, 3})\n    true","ref":"Oxide.Result.html#is_error?/1"},{"type":"function","title":"Oxide.Result.is_ok?/1","doc":"Return whether a result is ok.\n\n    iex> Result.is_ok?({:ok, 3})\n    true\n    iex> Result.is_ok?({:error, 3})\n    false","ref":"Oxide.Result.html#is_ok?/1"},{"type":"function","title":"Oxide.Result.map/2","doc":"Return a result leaving errors unchanged but transforming the value of an `:ok` result.\n\n    iex> Result.map({:ok, 3}, fn x -> x + 1 end)\n    {:ok, 4}\n    iex> Result.map({:error, :nan}, fn x -> x + 1 end)\n    {:error, :nan}","ref":"Oxide.Result.html#map/2"},{"type":"function","title":"Oxide.Result.map_err/2","doc":"Return a result leaving ok values unchanged but transforming an error reason with `f`.\n\n    iex> Result.map_err({:ok, 3}, fn x -> x + 1 end)\n    {:ok, 3}\n    iex> Result.map_err({:error, :nan}, &:erlang.atom_to_binary/1)\n    {:error, \"nan\"}","ref":"Oxide.Result.html#map_err/2"},{"type":"function","title":"Oxide.Result.map_or/3","doc":"Return an _unwrapped_ ok value transformed by f, or `default` if `result` is an error.\n\n    iex> Result.map_or({:ok, 3}, 0, fn x -> x + 1 end)\n    4\n    iex> Result.map_or({:error, :nan}, 0, fn x -> x + 1 end)\n    0","ref":"Oxide.Result.html#map_or/3"},{"type":"function","title":"Oxide.Result.ok/1","doc":"Wrap a value in an ok result.","ref":"Oxide.Result.html#ok/1"},{"type":"function","title":"Oxide.Result.or_else/2","doc":"Return an ok result unchanged, or transform an unwrapped error reason with `f`.\n\n    iex> Result.or_else({:ok, :xylophone}, fn err -> err + 1 end)\n    {:ok, :xylophone}\n    iex> Result.or_else({:error, 3}, fn err -> err + 1 end)\n    4","ref":"Oxide.Result.html#or_else/2"},{"type":"function","title":"Oxide.Result.tap_err/2","doc":"Equivalent to `Kernel.tap/2` for error results.\n\nCalls `f` with the reason of an `:error` result, and returns the result unchanged.\n\n    iex> {:ok, 3} |> Result.tap_err(&IO.inspect/1)\n    {:ok, 3}\n    iex> {:error, :oops} |> Result.tap_err(&IO.inspect/1)\n    :oops\n    {:error, :oops}","ref":"Oxide.Result.html#tap_err/2"},{"type":"function","title":"Oxide.Result.tap_ok/2","doc":"Equivalent to `Kernel.tap/2` for ok results.\n\nCalls `f` with the value of an `:ok` result, and returns the result unchanged.\n\n    iex> {:ok, 3} |> Result.tap_ok(&IO.inspect/1)\n    3\n    {:ok, 3}\n    iex> {:error, :oops} |> Result.tap_ok(&IO.inspect/1)\n    {:error, :oops}","ref":"Oxide.Result.html#tap_ok/2"},{"type":"function","title":"Oxide.Result.unwrap!/1","doc":"Unwrap an `:ok` result, and raise an `:error` reason.\n\nIf an error reason is an exception, it is raised as-is; otherwise, a `RuntimeError`\nis raised with the inspected error reason in the exception message.\n\n    iex> Result.unwrap!({:ok, :value})\n    :value\n    iex> Result.unwrap!({:error, %{code: 500}})\n    ** (RuntimeError) Unwrapped an error: %{code: 500}\n\n    iex> Result.unwrap!({:error, ArgumentError.exception(\"oh no\")})\n    ** (ArgumentError) oh no","ref":"Oxide.Result.html#unwrap!/1"},{"type":"function","title":"Oxide.Result.unwrap_err!/1","doc":"","ref":"Oxide.Result.html#unwrap_err!/1"},{"type":"function","title":"Oxide.Result.unwrap_or/2","doc":"Unwrap an `:ok` result, falling back to `default` for an `:error` result.\n\n    iex> Result.unwrap_or({:ok, :cake}, :icecream)\n    :cake\n    iex> Result.unwrap_or({:error, :peas}, :icecream)\n    :icecream","ref":"Oxide.Result.html#unwrap_or/2"},{"type":"function","title":"Oxide.Result.unwrap_or_else/2","doc":"Unwrap an `:ok` result, falling back to executing a zero-arity function if the result\nis an error.\n\n    iex> Result.unwrap_or_else({:ok, :cake}, fn -> :icecream end)\n    :cake\n    iex> Result.unwrap_or_else({:error, :peas}, fn -> :icecream end)\n    :icecream","ref":"Oxide.Result.html#unwrap_or_else/2"},{"type":"type","title":"Oxide.Result.t/0","doc":"","ref":"Oxide.Result.html#t:t/0"},{"type":"type","title":"Oxide.Result.t/1","doc":"","ref":"Oxide.Result.html#t:t/1"},{"type":"type","title":"Oxide.Result.t/2","doc":"","ref":"Oxide.Result.html#t:t/2"},{"type":"module","title":"Oxide.Result.Dangerous","doc":"Result helpers you should use with extreme care, or better yet, not at all.","ref":"Oxide.Result.Dangerous.html"},{"type":"macro","title":"Oxide.Result.Dangerous.~>/2","doc":"Dangerous result pipe operator.\n\nThe result pipe operator `~>/2` is defined identically to `Oxide.Result.&&&/2`, and in\nmany circumstances behaves the same:\n\n    iex> {:ok, :foo} ~> Atom.to_string()\n    \"foo\"\n    iex> {:ok, 3} ~> then(fn x -> {:ok, x + 1} end) ~> List.wrap()\n    [4]\n    iex> {:ok, :foo} ~> Atom.to_string() |> String.capitalize()\n    \"Foo\"\n    iex> {:error, :oops} ~> then(fn x -> {:ok, x + 1} end) ~> List.wrap()\n    {:error, :oops}\n\nHowever, Elixir's operator precedence means that when `~>` is followed by `|>`,\nan early error can pipe into later stages of the pipeline in a way that is\nalmost never what you want.\n\n    iex> {:error, :oops} &&& Atom.to_string() |> String.capitalize()\n    {:error, :oops}\n    iex> {:error, :oops} ~> Atom.to_string() |> String.capitalize()  # String.capitalize({:error, :oops})\n    ** (FunctionClauseError) no function clause matching in String.capitalize/2","ref":"Oxide.Result.Dangerous.html#~%3E/2"},{"type":"extras","title":"Oxide","doc":"# Oxide\n\nOxide is a library of helpers for working ergonomically with result tuples `{:ok, value}` and\n`{:error, reason}`. Most of its functions are direct equivalents to those in the Rust standard library,\nand it also introduces  `&&&` - a result-aware variant of the pipe operator\n`|>` - as an Elixir analogue to Rust's unary `?` operator.\n\n> ##","ref":"readme.html"},{"type":"extras","title":"Warning {: .warning} - Oxide","doc":">\n> Oxide expects results to be `{:ok, any()} | {:error, any()}`. In particular, `:ok`, `{:ok}` and\n> `{:ok, :foo, :bar}` are not results - instead use `{:ok, nil}`, `{:ok, nil}` and `{:ok, {:foo, :bar}}`\n> respectively. This is an intentional choice to encourage code to adopt a consistent approach\n> to typing with as little ambiguity or surprising behaviour as possible.","ref":"readme.html#warning-warning"},{"type":"extras","title":"Examples - Oxide","doc":"#","ref":"readme.html#examples"},{"type":"extras","title":"Pipelines of results - Oxide","doc":"```elixir\n# Before\nwith {:ok, x1}   f1() &&& f2() &&& f3()\n```\n\n#","ref":"readme.html#pipelines-of-results"},{"type":"extras","title":"Acting on :ok values - Oxide","doc":"```elixir\n# Act on the success value or leave errors unchanged.\n# Maps {:ok, n} -> {:ok, n + 1} and leaves {:error, reason} alone\nreturns_result() |> Result.map(fn val -> val + 1 end)\n```","ref":"readme.html#acting-on-ok-values"},{"type":"extras","title":"Doesn't `&&&` duplicate `with`? - Oxide","doc":"The developer ergonomics of `with` are a two-phase construct with a context initalization phase\n(`with  `) followed by an inner execution phase (`do   end`). It's extremely natural\nwhen there are a collection of independent preparatory steps followed by a distinct action or set of actions, as in this [great example](https://hexdocs.pm/elixir/1.16.1/Kernel.SpecialForms.html#with/1)\nfrom the Elixir docs:\n\n```elixir\ndef area(opts) do\n  with {:ok, width}   KVServer.Command.parse()\n|> KVServer.Command.run()\n```\n\nWe can't do that because we need to unwrap the results before passing them along the pipeline, and we want to bail early on error results.\n\nIn comparison with the pipeline, the `with` syntax is somewhat unnatural:\n\n* the `run` call lives in a different context to the preceding two\n* we have to start reading from right to left to follow the control flow\n* we must explicitly pass around arguments that would be elided in a pipeline\n* we've added a layer of nesting to our code\n\nThe point of `&&&` is to recover the natural pipeline expression:\n\n```elixir\nread_line(socket)\n&&& KVServer.Command.parse()\n&&& KVServer.Command.run()\n```","ref":"readme.html#doesn-t-duplicate-with"}],"content_type":"text/markdown"}